
// compile: g++-11.1 -std=c++20 icp-to_chars.cc
// source: https://en.cppreference.com/w/cpp/utility/to_chars

// std::to_chars ignores locale setting, does not alloc, is nothrow, and is fast

#include <iostream>
#include <charconv>
#include <system_error>
#include <string_view>
#include <array>

void show_to_chars(auto... format_args)
{
    std::array<char, 10> str;

    if (auto [ptr, ec]
            = std::to_chars(str.data(), str.data() + str.size(), format_args...);
        ec == std::errc())
        std::cout << std::string_view(str.data(), ptr) << '\n';
    else
        std::cout << std::make_error_code(ec).message() << '\n';
}

int main()
{
    show_to_chars(42);
    show_to_chars(+3.14159F);
    show_to_chars(-3.14159, std::chars_format::fixed);
    show_to_chars(-3.14159, std::chars_format::scientific, 3);
    show_to_chars(3.1415926535, std::chars_format::fixed, 10);

    std::array<char, 10> str;

    auto [ptr, ec] = std::to_chars(str.data(), str.data() + str.size(), 42);
    if(ec == std::errc())
         std::cout << std::string_view(str.data(), ptr); // C++20, uses string_view(first, last)
         //   (str.data(), ptr - str.data()); // C++17, uses string_view(ptr, length)
    std::cout << "\n";

}
